@@ -1,6 +1,15 @@
 package Graphics::Primitive::TextBox;
 use Moose;
 use MooseX::Storage;
+use Moose::Util::TypeConstraints;
+
+enum 'Graphics::Primitive::TextBox::Directions' => (
+    'auto', 'ltr', 'rtl'
+);
+enum 'Graphics::Primitive::TextBox::WrapModes'
+    => qw(word char word_char);
+enum 'Graphics::Primitive::TextBox::EllipsizeModes'
+    => qw(none start middle end);
 
 extends 'Graphics::Primitive::Component';
 
@@ -8,13 +17,22 @@ with qw(MooseX::Clone Graphics::Primitive::Aligned);
 with Storage (format => 'JSON', io => 'File');
 
 use Graphics::Primitive::Font;
-use Text::Flow;
 
 has 'angle' => (
     is => 'rw',
     isa => 'Num',
     trigger => sub { my ($self) = @_; $self->prepared(0); }
 );
+has 'direction' => (
+    is => 'rw',
+    isa => 'Graphics::Primitive::TextBox::Directions',
+    default => sub { 'auto' }
+);
+has 'ellipsize_mode' => (
+    is => 'rw',
+    isa => 'Graphics::Primitive::TextBox::EllipsizeModes',
+    default => sub { 'none' }
+);
 has 'font' => (
     is => 'rw',
     isa => 'Graphics::Primitive::Font',
@@ -22,16 +40,28 @@ has 'font' => (
     trigger => sub { my ($self) = @_; $self->prepared(0); }
 );
 has '+horizontal_alignment' => ( default => sub { 'left'} );
+has 'indent' => (
+    is => 'rw',
+    isa => 'Num',
+    default => sub { 0 }
+);
+has 'justify' => (
+    is => 'rw',
+    isa => 'Bool',
+    default => sub { 0 }
+);
+has 'layout' => (
+    is => 'rw',
+    does => 'Graphics::Primitive::Driver::TextLayout',
+);
+has 'lines' => (
+    is => 'rw',
+);
 has 'line_height' => (
     is => 'rw',
     isa => 'Num',
     trigger => sub { my ($self) = @_; $self->prepared(0); }
 );
-has 'lines' => (
-    is => 'rw',
-    isa => 'ArrayRef',
-    default => sub { [] }
-);
 has 'text' => (
     is => 'rw',
     isa => 'Str',
@@ -43,109 +73,24 @@ has 'text_bounding_box' => (
     trigger => sub { my ($self) = @_; $self->prepared(0); }
 );
 has '+vertical_alignment' => ( default => sub { 'top'} );
-
-override('finalize', sub {
-    my ($self, $driver) = @_;
-
-    super;
-
-    if(!scalar(@{ $self->lines }) && $self->text) {
-        $self->_layout_text($driver);
-    }
-});
+has 'wrap_mode' => (
+    is => 'rw',
+    isa => 'Graphics::Primitive::TextBox::WrapModes',
+    default => sub { 'word' }
+);
 
 override('prepare', sub {
     my ($self, $driver) = @_;
 
     super;
 
-    my $lh = $self->line_height;
-    unless(defined($lh)) {
-        $lh = $self->font->size;
-    }
-
-    # if($self->width && $self->height) {
-    #     $self->layout_text($driver);
-    # } else {
-    unless(scalar(@{ $self->lines })) {
-        my @lines = split("\n", $self->text);
-        my $twide = 0;
-        my $theight = 0;
-        foreach my $line (@lines) {
-            my ($bb, $tb)  = $driver->get_text_bounding_box(
-                $self->font, $line, $self->angle
-            );
-
-            $self->text_bounding_box($tb);
-            my $height = $bb->height;
-
-            if(defined($lh) && ($lh > $height)) {
-                $height = $lh;
-            }
-
-            push(@{ $self->lines }, { text => $line, box => $tb });
-            if($twide < $bb->width) {
-                $twide = $bb->width;
-            }
-            $theight += $height;
-        }
-        $self->minimum_height($theight + $self->minimum_height);
-        $self->minimum_width($twide + $self->outside_width);
-    }
-
+    my $layout = $driver->get_textbox_layout($self);
+    $self->layout($layout);
 
+    $self->minimum_height($layout->height + $self->outside_height);
+    $self->minimum_width($layout->width + $self->outside_width);
 });
 
-sub _layout_text {
-    my ($self, $driver) = @_;
-
-    # TODO Set a minimum size if we don't need the whole thing
-
-    my $size = 0;
-    my $flow = Text::Flow->new(
-        check_height => sub {
-            my $paras = shift;
-
-            my $lh = $self->line_height;
-            $lh = $self->font->size unless(defined($lh));
-
-            foreach my $p (@{ $paras }) {
-                if(defined($lh)) {
-                    $size += $lh * scalar(@{ $p });
-                }
-            }
-            if(($size + $lh) >= $self->inside_height) {
-                return 0;
-            }
-            return 1;
-        },
-        wrapper => Text::Flow::Wrap->new(
-            check_width => sub {
-                my $str = shift;
-                my $r = $driver->get_text_bounding_box(
-                    $self->font, $str
-                );
-                if($r->width > $self->inside_width) {
-                    return 0;
-                }
-                return 1;
-            }
-        )
-    );
-
-    my @text = $flow->flow($self->text);
-
-    my $para = $text[0];
-    my @lines = split(/\n/, $para);
-    foreach my $line (@lines) {
-        my ($cb, $tb) = $driver->get_text_bounding_box($self->font, $line);
-        push(@{ $self->lines }, {
-            text => $line,
-            box => $cb
-        });
-    }
-}
-
 __PACKAGE__->meta->make_immutable;
 
 1;
