@@ -18,6 +18,8 @@ sub _draw_arc { }
 
 sub _draw_canvas { }
 
+sub _draw_circle { }
+
 sub _draw_component {
     my ($self, $comp) = @_;
 
@@ -26,6 +28,8 @@ sub _draw_component {
     );
 }
 
+sub _draw_ellipse { }
+
 sub _draw_line { }
 
 sub _draw_path { }
@@ -44,6 +48,8 @@ sub data { }
 
 sub get_text_bounding_box { }
 
+sub get_textbox_layout { }
+
 sub reset { }
 
 sub write { }
